/**
 * 
 */
package de.mcs.tools.sps;

import java.util.List;

import de.mcs.tools.sps.exceptions.WrongProgramSizeException;

/**
 * @author w.klaas
 *
 */
public abstract class AbstractEmulator implements SPSEmulator {

  protected byte[] programMemory;
  private boolean debug;
  protected int address;
  protected int a, b, c, d;
  private boolean active;
  private TOutputCallback outputCallback;

  @Override
  public abstract List<SPSCommand> getCommands();

  @Override
  public abstract int getProgramSize();

  @Override
  public void loadProgram(byte[] program) throws WrongProgramSizeException {
    if (program.length > getProgramSize()) {
      throw new WrongProgramSizeException(
          String.format("the program is to long. Only %d bytes are permitted", getProgramSize()));
    }
    this.programMemory = program;
  }

  @Override
  public void startProgram(boolean debug) {
    this.debug = debug;
    reset();
    active = true;
    if (outputCallback != null) {
      outputCallback.onOutput(getEmulatorOutput());
    }
    if (!debug) {
      while (active) {
        callNextCommand();
      }
    }
  }

  protected abstract void callNextCommand();

  protected SPSCommandData getCommandData(byte command) {
    SPSCommandData commandData = null;
    byte commandByte = (byte) (command & (byte) 0xF0);

    List<SPSCommand> commands = getCommands();
    for (SPSCommand c : commands) {
      if (c.getCommandByte() == commandByte) {
        for (SPSCommandData cd : c.getCommandDatas()) {
          if (cd.getCommandByte() == command) {
            commandData = cd;
          }
        }
      }
    }
    return commandData;
  }

  protected void runCommand(SPSCommandData commandData) {
    commandData.doWork();
    if (outputCallback != null) {
      outputCallback.onOutput(getEmulatorOutput());
    }
  }

  protected abstract EmulatorOutput getEmulatorOutput();

  protected void reset() {
    address = 0;
    a = 0;
    b = 0;
    c = 0;
    d = 0;
  }

  @Override
  public void addCallback(TOutputCallback callback) {
    this.outputCallback = callback;
  }

  @Override
  public void nextStep() {
    callNextCommand();
  }

  @Override
  public boolean isActive() {
    return active;
  }

  @Override
  public void stop() {
    active = false;
  }

  @Override
  public EmulatorInternals getInternals() {
    return null;
  }

}
